import Alert from 'antd/lib/alert';
import Button from 'antd/lib/button';
import Form, { FormComponentProps } from 'antd/lib/form';
import Input from 'antd/lib/input';
import message from 'antd/lib/message';
import * as React from 'react';
import { connect } from 'react-redux';
import styled from 'styled-components';
import { useBoolean } from '../../../customHooks';
import { IState } from '../../../types';
import { authSetAuthDataAction, authSetAuthStateAction } from '../actions';
import { setNewPassword } from '../api';
import { AUTH_STATE_AUTHENTICATED } from '../constants';
import { IAuthData, selectAuthData } from '../reducer';

interface IProps extends FormComponentProps {
  setAuthState: any;
  setAuthData: any;
  authData: IAuthData;
}

const Actions = styled.div`
  display: flex;
  justify-content: flex-end;
`;

const SetNewPassword = ({
  form,
  authData,
  setAuthData,
  setAuthState,
}: IProps) => {
  const isLoading = useBoolean(false);
  const confirmDirty = useBoolean(false);

  const handleSubmit = (e: any) => {
    e.preventDefault();
    form.validateFields(async (err: any, values: any) => {
      if (!err) {
        try {
          isLoading.setTrue();
          const data = await setNewPassword(authData, values.password);
          await setAuthData(data);
          setAuthState(AUTH_STATE_AUTHENTICATED);
          message.success('Heslo úspěšně nastaveno');
        } catch (e) {
          if (e.code === 'InvalidPasswordException') {
            message.info(
              'Heslo musí být alespoň 8 znaků dlouhé a musí obsahovat čísla',
            );
          } else {
            message.error(e.message);
          }
        } finally {
          isLoading.setFalse();
        }
      }
    });
  };

  const handleConfirmBlur = (e: any) => {
    const value = e.target.value;
    confirmDirty.setValue(confirmDirty.value || !!value);
  };

  const compareToFirstPassword = (rule: any, value: any, callback: any) => {
    if (value && value !== form.getFieldValue('password')) {
      callback('Hesla se neshodují!');
    } else {
      callback();
    }
  };

  const validateToNextPassword = (rule: any, value: any, callback: any) => {
    if (value && confirmDirty.value) {
      form.validateFields(['confirm'], { force: true });
    }
    callback();
  };

  const { getFieldDecorator } = form;
  return (
    <Form onSubmit={handleSubmit}>
      <Alert
        message="Nyní si zvolte Vaše nové heslo."
        type="info"
        showIcon={true}
      />
      <br />
      <Form.Item>
        {getFieldDecorator('password', {
          rules: [
            {
              required: true,
              message: 'Prosím zadejte nové heslo!',
            },
            {
              validator: validateToNextPassword,
            },
          ],
        })(<Input.Password placeholder="Nové Heslo" />)}
      </Form.Item>
      <Form.Item>
        {getFieldDecorator('confirm', {
          rules: [
            {
              required: true,
              message: 'Vyplňte prosím potvrzení hesla!',
            },
            {
              validator: compareToFirstPassword,
            },
          ],
        })(
          <Input.Password
            placeholder="Potvrzení Hesla"
            onBlur={handleConfirmBlur}
          />,
        )}
      </Form.Item>
      <Form.Item>
        <Actions>
          <Button type="primary" htmlType="submit" loading={isLoading.value}>
            Potvrdit
          </Button>
        </Actions>
      </Form.Item>
    </Form>
  );
};

export default connect(
  (state: IState) => ({
    authData: selectAuthData(state),
  }),
  {
    setAuthState: authSetAuthStateAction,
    setAuthData: authSetAuthDataAction,
  },
)(Form.create()(SetNewPassword));
